@@ -26,6 +26,15 @@ st_dimensions.dimensions = function(.x, ...) .x
 	st_stars(x, value)
 }
 
+#' @export
+#' @name st_dimensions
+`st_dimensions<-.stars_proxy` = function(x, value) {
+	if (!is.null(attr(x, "call_list")))
+		stop("st_dimensions<- on a stars_proxy object only works if there is no call list")
+	NextMethod()
+}
+
+
 #' @export
 #' @name st_dimensions
 `st_dimensions<-.list` = function(x, value) {
@@ -125,26 +134,34 @@ st_set_dimensions = function(.x, which, values = NULL, point = NULL, names = NUL
 		d[[which]] = create_dimension(values = values, point = point %||% d[[which]]$point, ...)
 		if (! is.null(names) && length(names) == 1)
 			base::names(d)[which] = names
+		r = attr(d, "raster")
+		if (isTRUE(r$curvilinear)) {
+			# FIXME: there's much more that should be checked for curvilinear grids...
+			# https://github.com/r-spatial/stars/issues/460
+			if (which %in% r$dimensions && !is.matrix(values))
+				attr(d, "raster")$curvilinear = FALSE 
+		}
 #		else if (inherits(values, "sfc"))
 #			base::names(d)[which] = "sfc"
 	} else if (!is.null(point) && is.logical(point)) {
 		d[[which]]$point = point
 	} else if (!missing(names)) {
-		# handle names in raster attribute, #46
-		r = attr(d, "raster")
-		if (any(!is.na(r$dimensions))) {
-			r$dimensions = names[match(r$dimensions, names(d))]
-			attr(d, "raster") = r
-		}
 		if (length(d) != length(names) && length(names) != 1)
 			stop("length of names should match number of dimensions")
-		new_names = if (length(d) == length(names))
-				names
-			else { # replace the name of dimension `which`, #354
-				new_names = names(d)
-				new_names[which] = names
-				new_names
-			}
+		r = attr(d, "raster")
+		if (length(d) == length(names)) {
+			# handle names in raster attribute, #46
+			# problematic again in #379
+			if (any(!is.na(r$dimensions)))
+				r$dimensions = names[match(r$dimensions, names(d))]
+			new_names = names
+		} else { # replace the name of dimension `which`, #354
+			if (names(d)[which] %in% r$dimensions)
+				r$dimensions[match(names(d)[which], r$dimensions)] = names
+			new_names = names(d)
+			new_names[which] = names
+		}
+		attr(d, "raster") = r
 		base::names(d) = new_names
 	} else if (! missing(xy)) {
 		stopifnot(length(xy) == 2)
@@ -161,12 +178,38 @@ st_set_dimensions = function(.x, which, values = NULL, point = NULL, names = NUL
 
 
 #' @name st_dimensions
+#' @param where character, one of 'start', 'center' or 'end'. Set to NA (default) to ignore and use \code{max} and \code{center} explictly.  This argument provides a convenient alternative to setting \code{max} and \code{center}.
 #' @param max logical; if \code{TRUE} return the end, rather than the beginning of an interval
 #' @param center logical; if \code{TRUE} return the center of an interval; if \code{NA} return the center for raster dimensions, and the start of intervals in other cases
 #' @export
-st_get_dimension_values = function(.x, which, ..., max = FALSE, center = NA) {
+#' @examples
+#' m = matrix(1:20, nrow = 5, ncol = 4)
+#' dim(m) = c(x = 5, y = 4) # named dim
+#' (s = st_as_stars(m))
+#' st_get_dimension_values(s, 'x', where = "start")
+#' st_get_dimension_values(s, 'x', center = FALSE)
+#' st_get_dimension_values(s, 'x', where = "center")
+#' st_get_dimension_values(s, 'x', center = TRUE)
+#' st_get_dimension_values(s, 'x', where = "end")
+#' st_get_dimension_values(s, 'x', max = TRUE)
+st_get_dimension_values = function(.x, which, ..., where = NA, max = FALSE, center = NA) {
 	if ((!is.numeric(which) && !is.character(which)) || length(which) != 1)
 		stop("argument which should be a length 1 dimension index or name") # nocov
+  if (!is.na(where)){
+    w = tolower(where[1])
+    if (w == 'center'){
+      max = FALSE
+      center = TRUE
+    } else if (w == 'start'){
+      max = FALSE
+      center = FALSE
+    } else if (w == 'end'){
+      max = TRUE
+      center = NA
+    } else {
+      stop("where, if not NA, must be 'start', 'center' or 'end': ", where)
+    }
+  }
 	expand_dimensions(.x, ..., max = max, center = center)[[which]]
 }
 
@@ -193,7 +236,7 @@ regular_intervals = function(x, epsilon = 1e-10) {
 				else
 					return(FALSE)
 			}
-		isTRUE(abs(diff(range(ud)) / mean(ud)) < epsilon)
+		isTRUE(as.numeric(abs(diff(range(ud)) / mean(ud))) < epsilon)
 	}
 }
 
@@ -271,8 +314,6 @@ create_dimensions = function(lst, raster = NULL) {
 get_crs = function(pr) {
 	if (!is.null(pr$crs))
 		pr$crs
-	else if (!is.null(pr$proj4string)) # older sf
-		st_crs(pr$proj4string)
 	else if (!is.null(pr$wkt)) # newer sf, but GDAL < 3.0.0
 		st_crs(pr$wkt)
 	else if (!is.null(pr$proj_wkt))
@@ -359,6 +400,7 @@ print.stars_raster = function(x, ...) {
 		cat(paste("sheared raster with parameters:", x$affine[1], x$affine[2], "\n"))
 	if (x$curvilinear)
 		cat("curvilinear grid\n")
+	invisible(x)
 }
 
 get_val = function(pattern, meta) {
@@ -376,6 +418,8 @@ parse_netcdf_meta = function(pr, name) {
 			"zzzzz40163a99980" # bogus string, to avoid match
 		else
 			tail(spl, 1) # last one contains 
+	# longname:
+	pr$long_name = get_val(paste0(get_val("NC_GLOBAL#variable_id", meta), "#long_name"), meta)
 	# unit:
 	pr$units = get_val(paste0(name, "#units"), meta)
 	# extra dims: NETCDF_DIM_EXTRA={time,zlev}
@@ -519,9 +563,8 @@ dim.dimensions = function(x) {
 		lengths(expand_dimensions(x)) # FIXME: optimise?
 }
 
-
 #' @export
-print.dimensions = function(x, ..., digits = 6, usetz = TRUE) {
+as.data.frame.dimensions = function(x, ..., digits = 6, usetz = TRUE, stars_crs = getOption("stars.crs") %||% 28) {
 	lst = lapply(x, function(y) {
 			if (length(y$values) > 3) {
 				y$values = if (is.array(y$values))
@@ -536,8 +579,8 @@ print.dimensions = function(x, ..., digits = 6, usetz = TRUE) {
 			}
 			if (is.na(y$refsys))
 				y$refsys = NA_character_
-			else if (nchar(tail(format(y$refsys), 1)) > 28)
-				y$refsys = paste0(substr(tail(format(y$refsys), 1), 1L, 25),"...")
+			else if (nchar(tail(format(y$refsys), 1)) > stars_crs)
+				y$refsys = paste0(substr(tail(format(y$refsys), 1), 1L, stars_crs - 3),"...")
 			y
 		}
 	)
@@ -550,18 +593,24 @@ print.dimensions = function(x, ..., digits = 6, usetz = TRUE) {
 	lst = lapply(lst, function(x) sapply(x, mformat, digits = digits))
 	ret = data.frame(do.call(rbind, lst), stringsAsFactors = FALSE)
 	r = attr(x, "raster")
-	if (!any(is.na(r$dimensions))) {
+	if (! any(is.na(r$dimensions))) {
 		ret$raster = rep("", nrow(ret))
 		ret[r$dimensions[1], "raster"] = "[x]"
 		ret[r$dimensions[2], "raster"] = "[y]"
 		names(ret) = c(names(lst[[1]]), "x/y")
 	}
-	print(ret)
-	print(attr(x, "raster"))
-	invisible(ret)
+	ret
 }
 
-identical_dimensions = function(lst, ignore_resolution = FALSE) {
+#' @export
+print.dimensions = function(x, ...) {
+	ret = as.data.frame(x, ...)
+	print(ret)
+	print(attr(x, "raster"))
+	invisible(x)
+}
+
+identical_dimensions = function(lst, ignore_resolution = FALSE, tolerance = 0) {
 	if (length(lst) > 1) {
 		d1 = attr(lst[[1]], "dimensions")
 		for (i in 2:length(lst)) {
@@ -572,7 +621,7 @@ identical_dimensions = function(lst, ignore_resolution = FALSE) {
 				for (j in seq_along(di))
 					di[[j]]$delta = di[[j]]$to = NA_real_
 			}
-			if (! isTRUE(all.equal(d1, di)))
+			if (! isTRUE(all.equal(d1, di, tolerance = tolerance)))
 				return(FALSE)
 		}
 	}
