@@ -81,7 +81,7 @@ transform_grid_grid = function(x, target) {
 	dxy = attr(target, "raster")$dimensions
 
 	from = st_crs(target)
-	pts = sf_project(from = from, to = st_crs(x), pts = new_pts)
+	pts = sf::sf_project(from = from, to = st_crs(x), pts = new_pts)
 
 	# at xy (target) locations, get values from x, or put NA
 	# to array:
@@ -119,7 +119,8 @@ transform_grid_grid = function(x, target) {
 #' @param segments (total) number of segments for segmentizing the bounding box before transforming to the new crs
 #' @param use_gdal logical; if \code{TRUE}, use gdalwarp, through \link[sf]{gdal_utils}
 #' @param options character vector with options, passed on to gdalwarp
-#' @param no_data_value value used by gdalwarp for no_data (NA) when writing to temporary file
+#' @param no_data_value value used by gdalwarp for no_data (NA) when writing to temporary file; 
+#'  not setting this when \code{use_gdal} is \code{TRUE} leads to a warning
 #' @param debug logical; if \code{TRUE}, do not remove the temporary gdalwarp destination file, and print its name
 #' @param method character; see details for options; methods other than \code{near} only work when \code{use_gdal=TRUE}
 #' @param ... ignored
@@ -139,6 +140,10 @@ transform_grid_grid = function(x, target) {
 #' r %>% st_set_crs(4326) %>% st_warp(st_as_stars(st_bbox(), dx = 2)) -> s
 #' plot(r, axes = TRUE) # no CRS set, so no degree symbols in labels
 #' plot(s, axes = TRUE)
+#' # downsample raster (90 to 270 m)
+#' r = read_stars(system.file("tif/olinda_dem_utm25s.tif", package = "stars"))
+#' r270 = st_as_stars(st_bbox(r), dx = 270)
+#' r270 = st_warp(r, r270)
 #' @details For gridded spatial data (dimensions \code{x} and \code{y}), see figure; the existing grid is transformed into a regular grid defined by \code{dest}, possibly in a new coordinate reference system. If \code{dest} is not specified, but \code{crs} is, the procedure used to choose a target grid is similar to that of \link[raster]{projectRaster} (currently only with \code{method='ngb'}). This entails: (i) the envelope (bounding box polygon) is transformed into the new crs, possibly after segmentation (red box); (ii) a grid is formed in this new crs, touching the transformed envelope on its East and North side, with (if cellsize is not given) a cellsize similar to the cell size of \code{src}, with an extent that at least covers \code{x}; (iii) for each cell center of this new grid, the matching grid cell of \code{x} is used; if there is no match, an \code{NA} value is used.
 #' @export
 st_warp = function(src, dest, ..., crs = NA_crs_, cellsize = NA_real_, segments = 100,
@@ -151,11 +156,18 @@ st_warp = function(src, dest, ..., crs = NA_crs_, cellsize = NA_real_, segments
 	if (!is.na(crs))
 		crs = st_crs(crs)
 
+	if (!missing(dest) && inherits(dest, "crs"))
+		stop("target crs should be specified with crs = ..., not as argument dest")
+
 	ret = if (use_gdal) {
-		options = c(options, "-dstnodata", no_data_value, "-r", method)
+		if (!is.na(no_data_value))
+			options = c(options, "-dstnodata", no_data_value)
+		else 
+			warning("no_data_value not set: missing values will appear as zero values")
+		options = c(options, "-r", method)
 		if (all(!is.na(cellsize))) {
 			cellsize = rep(abs(cellsize), length.out = 2)
-			options = c(options, "-ts", cellsize[1], cellsize[2])
+			options = c(options, "-tr", cellsize[1], cellsize[2])
 		}
 		if (! inherits(src, "stars_proxy")) {
 			src = st_as_stars_proxy(src, NA_value = no_data_value)
@@ -164,11 +176,20 @@ st_warp = function(src, dest, ..., crs = NA_crs_, cellsize = NA_real_, segments
 			else
 				on.exit(unlink(src[[1]])) # temp file
 		}
+		# collapse a multi-file proxy objects into single-file/multi-band:
+		if (length(src[[1]]) > 1 || length(src) > 1) {
+			tmp = tempfile(fileext = ".vrt") # multi-band destination
+			sf::gdal_utils("buildvrt", unlist(src), tmp, options = "-separate")
+			src[[1]] = tmp
+		}
 		if (missing(dest) && is.na(crs)) {
 			delete = TRUE
 			dest = tempfile(fileext = ".tif")
 			sf::gdal_utils("warp", src[[1]], dest, options = options)
 		} else {  # dest exists, and should be used: should use warper rather than warp
+			# https://github.com/r-spatial/stars/issues/407
+			if (length(dim(src)) == 3 && length(dim(dest)) == 2)
+				dest = merge(do.call(c, lapply(seq_len(dim(src)[3]), function(x) dest)))
 			dest = if (! inherits(dest, "stars_proxy")) {
 					dest[[1]] = NA_real_ * dest[[1]] # blank out values
 					delete = !debug
