@@ -6,7 +6,7 @@ tif = system.file("tif/L7_ETMs.tif", package = "stars")
 (y <- x %>% select(L7_ETMs.tif))
 (y <- x %>% filter(band > 2))
 (z <- x %>% mutate(foo = 2* L7_ETMs.tif))
-x %>% pull(L7_ETMs.tif) %>% head
+x %>% pull(L7_ETMs.tif) %>% class()
 x %>% filter(x > 289900)
 x %>% slice("x", 50:100)
 
